#! /bin/sh

Usage()
{
	echo "-h : Prints this message"
	echo "-b : Selects boot device (hsmmc/nand)"
	echo "-c : Selects configblock partition only update"
	echo "-d : Selects debug boot loader without flashing"
	echo "-k : Selects kernel partition only update"
	echo "-r : Selects RAM size (256/512)"
	echo "-u : Selects boot loader partition only update"
}

BOOT_DEVICE=nand
CFGBLOCK_ONLY=0
DEBUG_ONLY=0
KERNEL_ONLY=0
RAM_SIZE=256
UBOOT_ONLY=0

while getopts "b:cdhkr:u" Option ; do
	case $Option in
		h) Usage
			# Exit if only usage (-h) was specfied.
			if [[ $# -eq 1 ]] ; then
				exit 10
			fi
			exit 0
			;;
		b) BOOT_DEVICE=$OPTARG
			;;
		c) CFGBLOCK_ONLY=1
			;;
		d) DEBUG_ONLY=1
			;;
		k) KERNEL_ONLY=1
			;;
		r) RAM_SIZE=$OPTARG
			;;
		u) UBOOT_ONLY=1
			;;
	esac
done

if [ ${DEBUG_ONLY} -eq 1 ] ; then
	# Debugging only
	sudo LD_LIBRARY_PATH="$(pwd)/nvflash" nvflash/nvflash --bl bin/u-boot.bin --go
	exit 0
fi

cd bin

if [ ${CFGBLOCK_ONLY} -ne 1 ] && [ ${KERNEL_ONLY} -ne 1 ] && [ ${UBOOT_ONLY} -ne 1 ] ; then
	# Prepare full flashing
	if [ ${BOOT_DEVICE} = "nand" ] ; then
		sudo ../nvflash/mkyaffs2image ../rootfs USR.yaffs2 4k512
	fi

	if [ ${BOOT_DEVICE} = "hsmmc" ] ; then
		sudo ./genext2fs -d /srv/nfs/rootfs -b 500000 APP.ext2
	fi

	# Full flashing
	sudo LD_LIBRARY_PATH="$(pwd)/../nvflash" ../nvflash/nvflash --bct ../ColibriT20_333_${RAM_SIZE}_${BOOT_DEVICE}.bct --setbct --configfile  ../lnx_${BOOT_DEVICE}.cfg --create --bl fastboot.bin --go

	# Cleanup after full flashing
	if [ ${BOOT_DEVICE} = "hsmmc" ] ; then
		sudo rm APP.ext2
	fi

	if [ ${BOOT_DEVICE} = "nand" ] ; then
		sudo rm USR.yaffs2
	fi
else
	# Partial flashing
	if ([ ${CFGBLOCK_ONLY} -eq 1 ] && [ ${KERNEL_ONLY} -ne 1 ] && [ ${UBOOT_ONLY} -ne 1 ]) || ([ ${CFGBLOCK_ONLY} -ne 1 ] && [ ${KERNEL_ONLY} -eq 1 ] && [ ${UBOOT_ONLY} -ne 1 ]) || ([ ${CFGBLOCK_ONLY} -ne 1 ] && [ ${KERNEL_ONLY} -ne 1 ] && [ ${UBOOT_ONLY} -eq 1 ]) ; then
		if [ ${UBOOT_ONLY} -eq 1 ] ; then
			# U-Boot flashing
			sudo LD_LIBRARY_PATH="$(pwd)/../nvflash" ../nvflash/nvflash --bl fastboot.bin --download 4 u-boot.bin
		fi
		if [ ${KERNEL_ONLY} -eq 1 ] ; then
			# Linux kernel flashing
			sudo LD_LIBRARY_PATH="$(pwd)/../nvflash" ../nvflash/nvflash --bl fastboot.bin --download 7 uImage
		fi
		if [ ${CFGBLOCK_ONLY} -eq 1 ] ; then
			# Configuration block flashing
			sudo LD_LIBRARY_PATH="$(pwd)/../nvflash" ../nvflash/nvflash --bl fastboot.bin --download 8 configblock_${RAM_SIZE}.bin
		fi
	else
		echo "Only one of -c, -k or -u is possible at any one time!"
	fi
fi
